@@ -6,6 +6,8 @@ pub use build::AckFrameBuilder;
 
 use super::*;
 
+const FRAME_HEADER_SIZE: usize = 1;
+
 const CONNECT_FRAME_ID: u8 = 0;
 const CONNECT_ACK_FRAME_ID: u8 = 1;
 const DISCONNECT_FRAME_ID: u8 = 2;
@@ -14,28 +16,29 @@ const DATA_FRAME_ID: u8 = 4;
 const SYNC_FRAME_ID: u8 = 5;
 const ACK_FRAME_ID: u8 = 6;
 
-const DATAGRAM_HEADER_SIZE_FRAGMENT: usize = 15;
-const DATAGRAM_HEADER_SIZE_FULL: usize = 11;
-
-const FRAME_ACK_SIZE: usize = 9;
-
 const CONNECT_FRAME_PAYLOAD_SIZE: usize = 18;
+
 const CONNECT_ACK_FRAME_PAYLOAD_SIZE: usize = 4;
 
 const DISCONNECT_FRAME_PAYLOAD_SIZE: usize = 0;
+
 const DISCONNECT_ACK_FRAME_PAYLOAD_SIZE: usize = 0;
 
+const DATAGRAM_HEADER_SIZE_FRAGMENT: usize = 15;
+const DATAGRAM_HEADER_SIZE_FULL: usize = 11;
 const DATA_FRAME_PAYLOAD_HEADER_SIZE: usize = 7;
+pub const MAX_DATAGRAM_OVERHEAD: usize = DATAGRAM_HEADER_SIZE_FRAGMENT;
+pub const DATA_FRAME_OVERHEAD: usize = FRAME_HEADER_SIZE + DATA_FRAME_PAYLOAD_HEADER_SIZE;
 
 const SYNC_FRAME_PAYLOAD_SIZE: usize = 9;
+pub const SYNC_FRAME_SIZE: usize = FRAME_HEADER_SIZE + SYNC_FRAME_PAYLOAD_SIZE;
 
-const ACK_FRAME_PAYLOAD_HEADER_SIZE: usize = 6;
+const ACK_GROUP_SIZE: usize = 9;
+const ACK_FRAME_PAYLOAD_HEADER_SIZE: usize = 10;
+pub const ACK_FRAME_OVERHEAD: usize = FRAME_HEADER_SIZE + ACK_FRAME_PAYLOAD_HEADER_SIZE;
 
-pub use build::MAX_CHANNELS;
+pub const MAX_CHANNELS: usize = 64;
 pub const MAX_FRAGMENTS: usize = 1 << 16;
-pub const MAX_DATAGRAM_OVERHEAD: usize = DATAGRAM_HEADER_SIZE_FRAGMENT;
-pub const DATA_FRAME_OVERHEAD: usize = 1 + DATA_FRAME_PAYLOAD_HEADER_SIZE;
-pub const SYNC_FRAME_SIZE: usize = 1 + SYNC_FRAME_PAYLOAD_SIZE;
 
 fn read_connect_payload(data: &[u8]) -> Option<Frame> {
     if data.len() != CONNECT_FRAME_PAYLOAD_SIZE {
@@ -105,152 +108,6 @@ fn read_disconnect_ack_payload(data: &[u8]) -> Option<Frame> {
     Some(Frame::DisconnectAckFrame(DisconnectAckFrame { }))
 }
 
-/*
-fn read_message_message(data: &[u8]) -> Option<(Message, usize)> {
-    if data.len() < MESSAGE_HEADER_SIZE {
-        return None;
-    }
-
-    let header_byte = data[0];
-
-    if header_byte & 0x80 == 0x80 {
-        // Datagram
-        if header_byte & 0x40 == 0x40 {
-            // Fragment
-            let header_size = DATAGRAM_MESSAGE_HEADER_SIZE_FRAGMENT;
-
-            if data.len() < header_size {
-                return None;
-            }
-
-            let channel_id = header_byte & 0x3F;
-
-            let sequence_id = ((data[1] as u32) << 24) |
-                              ((data[2] as u32) << 16) |
-                              ((data[3] as u32) <<  8) |
-                              ((data[4] as u32)      );
-
-            let window_parent_lead = ((data[5] as u16) << 8) |
-                                     ((data[6] as u16)     );
-
-            let channel_parent_lead = ((data[7] as u16) << 8) |
-                                      ((data[8] as u16)     );
-
-            let fragment_id_last = ((data[ 9] as u16) << 8) |
-                                   ((data[10] as u16)     );
-
-            let fragment_id = ((data[11] as u16) << 8) |
-                              ((data[12] as u16)     );
-
-            let data_len_u16 = ((data[13] as u16) << 8) |
-                               ((data[14] as u16)     );
-            let data_len = data_len_u16 as usize;
-
-            if data.len() < header_size + data_len {
-                return None;
-            }
-
-            let data = data[header_size .. header_size + data_len].into();
-
-            return Some((Message::Datagram(Datagram {
-                channel_id,
-                sequence_id,
-                window_parent_lead,
-                channel_parent_lead,
-                fragment_id: FragmentId { id: fragment_id, last: fragment_id_last },
-                data,
-            }), header_size + data_len));
-        } else {
-            // Full
-            let header_size = DATAGRAM_MESSAGE_HEADER_SIZE_FULL;
-
-            if data.len() < header_size {
-                return None;
-            }
-
-            let channel_id = header_byte & 0x3F;
-
-            let sequence_id = ((data[1] as u32) << 24) |
-                              ((data[2] as u32) << 16) |
-                              ((data[3] as u32) <<  8) |
-                              ((data[4] as u32)      );
-
-            let window_parent_lead = ((data[5] as u16) << 8) |
-                                     ((data[6] as u16)     );
-
-            let channel_parent_lead = ((data[7] as u16) << 8) |
-                                      ((data[8] as u16)     );
-
-            let data_len_u16 = ((data[ 9] as u16) << 8) |
-                               ((data[10] as u16)     );
-            let data_len = data_len_u16 as usize;
-
-            if data.len() < header_size + data_len {
-                return None;
-            }
-
-            let data = data[header_size .. header_size + data_len].into();
-
-            return Some((Message::Datagram(Datagram {
-                channel_id,
-                sequence_id,
-                window_parent_lead,
-                channel_parent_lead,
-                fragment_id: FragmentId { id: 0, last: 0 },
-                data,
-            }), header_size + data_len));
-        }
-    } else if header_byte & 0xC0 == 0x00 {
-        // Ack
-        if data.len() < ACK_MESSAGE_SIZE {
-            return None;
-        }
-
-        let frame_ack_nonce = header_byte & 0x01 == 0x01;
-
-        let frame_ack_base_id = ((data[1] as u32) << 24) |
-                                ((data[2] as u32) << 16) |
-                                ((data[3] as u32) <<  8) |
-                                ((data[4] as u32)      );
-
-        let frame_ack_bitfield = ((data[5] as u32) << 24) |
-                                 ((data[6] as u32) << 16) |
-                                 ((data[7] as u32) <<  8) |
-                                 ((data[8] as u32)      );
-
-        let receiver_base_id = ((data[ 9] as u32) << 24) |
-                               ((data[10] as u32) << 16) |
-                               ((data[11] as u32) <<  8) |
-                               ((data[12] as u32)      );
-
-        return Some((Message::Ack(Ack {
-            frames: FrameAck {
-                base_id: frame_ack_base_id,
-                bitfield: frame_ack_bitfield,
-                nonce: frame_ack_nonce,
-            },
-            receiver_base_id,
-        }), ACK_MESSAGE_SIZE));
-    } else if header_byte & 0xC0 == 0x40 {
-        // Resync
-        if data.len() < RESYNC_MESSAGE_SIZE {
-            return None;
-        }
-
-        let sender_next_id = ((data[1] as u32) << 24) |
-                             ((data[2] as u32) << 16) |
-                             ((data[3] as u32) <<  8) |
-                             ((data[4] as u32)      );
-
-        return Some((Message::Resync(Resync {
-            sender_next_id,
-        }), RESYNC_MESSAGE_SIZE));
-    }
-
-    None
-}
-*/
-
 fn read_datagram(data: &[u8]) -> Option<(Datagram, usize)> {
     if data.len() < 1 {
         return None;
@@ -386,24 +243,32 @@ fn read_sync_payload(data: &[u8]) -> Option<Frame> {
         return None;
     }
 
-    let sequence_id = ((data[0] as u32) << 24) |
-                      ((data[1] as u32) << 16) |
-                      ((data[2] as u32) <<  8) |
-                      ((data[3] as u32)      );
+    let mode = data[0];
 
-    let nonce = data[4] != 0;
+    let next_frame_id = if mode & 0x01 != 0x00 {
+        Some(((data[1] as u32) << 24) |
+             ((data[2] as u32) << 16) |
+             ((data[3] as u32) <<  8) |
+             ((data[4] as u32)      ))
+    } else {
+        None
+    };
 
-    let sender_next_id = ((data[5] as u32) << 24) |
-                         ((data[6] as u32) << 16) |
-                         ((data[7] as u32) <<  8) |
-                         ((data[8] as u32)      );
+    let next_packet_id = if mode & 0x02 != 0x00 {
+        Some(((data[5] as u32) << 24) |
+             ((data[6] as u32) << 16) |
+             ((data[7] as u32) <<  8) |
+             ((data[8] as u32)      ))
+    } else {
+        None
+    };
 
-    Some(Frame::SyncFrame(SyncFrame { sequence_id, nonce, sender_next_id }))
+    Some(Frame::SyncFrame(SyncFrame { next_frame_id, next_packet_id }))
 }
 
-fn read_frame_ack(data: &[u8]) -> Option<(FrameAck, usize)> {
+fn read_frame_ack(data: &[u8]) -> Option<(AckGroup, usize)> {
     // Ack
-    if data.len() < FRAME_ACK_SIZE {
+    if data.len() < ACK_GROUP_SIZE {
         return None;
     }
 
@@ -419,11 +284,11 @@ fn read_frame_ack(data: &[u8]) -> Option<(FrameAck, usize)> {
 
     let nonce = data[8] != 0;
 
-    return Some((FrameAck {
+    return Some((AckGroup {
         base_id,
         bitfield,
         nonce,
-    }, FRAME_ACK_SIZE));
+    }, ACK_GROUP_SIZE));
 }
 
 fn read_ack_payload(data: &[u8]) -> Option<Frame> {
@@ -431,13 +296,18 @@ fn read_ack_payload(data: &[u8]) -> Option<Frame> {
         return None;
     }
 
-    let receiver_base_id = ((data[0] as u32) << 24) |
-                           ((data[1] as u32) << 16) |
-                           ((data[2] as u32) <<  8) |
-                           ((data[3] as u32)      );
+    let frame_window_base_id = ((data[0] as u32) << 24) |
+                               ((data[1] as u32) << 16) |
+                               ((data[2] as u32) <<  8) |
+                               ((data[3] as u32)      );
 
-    let frame_ack_num = ((data[4] as u16) << 8) |
-                        ((data[5] as u16)     );
+    let packet_window_base_id = ((data[4] as u32) << 24) |
+                                ((data[5] as u32) << 16) |
+                                ((data[6] as u32) <<  8) |
+                                ((data[7] as u32)      );
+
+    let frame_ack_num = ((data[8] as u16) << 8) |
+                        ((data[9] as u16)     );
 
     let mut data_slice = &data[ACK_FRAME_PAYLOAD_HEADER_SIZE..];
     let mut frame_acks = Vec::new();
@@ -455,7 +325,7 @@ fn read_ack_payload(data: &[u8]) -> Option<Frame> {
         return None;
     }
 
-    Some(Frame::AckFrame(AckFrame { receiver_base_id, frame_acks }))
+    Some(Frame::AckFrame(AckFrame { frame_window_base_id, packet_window_base_id, frame_acks }))
 }
 
 
@@ -509,29 +379,35 @@ fn write_data(frame: &DataFrame) -> Box<[u8]> {
     let mut builder = build::DataFrameBuilder::new(frame.sequence_id, frame.nonce);
 
     for datagram in frame.datagrams.iter() {
-        builder.add(datagram);
+        builder.add(&datagram.into());
     }
 
     builder.build()
 }
 
 fn write_sync(frame: &SyncFrame) -> Box<[u8]> {
+    let mode = ((frame.next_frame_id.is_some() as u8) << 0) |
+               ((frame.next_packet_id.is_some() as u8) << 1);
+
+    let next_frame_id = frame.next_frame_id.unwrap_or(0);
+    let next_packet_id = frame.next_packet_id.unwrap_or(0);
+
     Box::new([
         SYNC_FRAME_ID,
-        (frame.sequence_id >> 24) as u8,
-        (frame.sequence_id >> 16) as u8,
-        (frame.sequence_id >>  8) as u8,
-        (frame.sequence_id      ) as u8,
-        frame.nonce as u8,
-        (frame.sender_next_id >> 24) as u8,
-        (frame.sender_next_id >> 16) as u8,
-        (frame.sender_next_id >>  8) as u8,
-        (frame.sender_next_id      ) as u8,
+        mode,
+        (next_frame_id >> 24) as u8,
+        (next_frame_id >> 16) as u8,
+        (next_frame_id >>  8) as u8,
+        (next_frame_id      ) as u8,
+        (next_packet_id >> 24) as u8,
+        (next_packet_id >> 16) as u8,
+        (next_packet_id >>  8) as u8,
+        (next_packet_id      ) as u8,
     ])
 }
 
 fn write_ack(frame: &AckFrame) -> Box<[u8]> {
-    let mut builder = build::AckFrameBuilder::new(frame.receiver_base_id);
+    let mut builder = build::AckFrameBuilder::new(frame.frame_window_base_id, frame.packet_window_base_id);
 
     for frame_ack in frame.frame_acks.iter() {
         builder.add(frame_ack);
@@ -581,10 +457,10 @@ mod tests {
     use super::*;
 
     fn verify_consistent(f: &Frame) {
-        //println!("frame: {:#?}", f);
+        println!("frame: {:#?}", f);
 
         let bytes = f.write();
-        //println!("frame bytes: {:?}", bytes);
+        println!("frame bytes: {:?}", bytes);
 
         let f2 = Frame::read(&bytes).unwrap();
 
@@ -692,9 +568,15 @@ mod tests {
     #[test]
     fn sync_basic() {
         let f = Frame::SyncFrame(SyncFrame {
-            sequence_id: 0x010203,
-            nonce: true,
-            sender_next_id: 0x12345678,
+            next_frame_id: Some(0x01020304),
+            next_packet_id: None,
+        });
+        verify_consistent(&f);
+        verify_extra_bytes_fail(&f);
+        verify_truncation_fails(&f);
+        let f = Frame::SyncFrame(SyncFrame {
+            next_frame_id: None,
+            next_packet_id: Some(0x05060708),
         });
         verify_consistent(&f);
         verify_extra_bytes_fail(&f);
@@ -704,9 +586,10 @@ mod tests {
     #[test]
     fn ack_basic() {
         let f = Frame::AckFrame(AckFrame {
-            receiver_base_id: 0x010203,
+            frame_window_base_id: 0x010203,
+            packet_window_base_id: 0x040506,
             frame_acks: vec![
-                FrameAck {
+                AckGroup {
                     base_id: 0x28475809,
                     bitfield: 0b01000100111101110110100110101u32,
                     nonce: true,
@@ -733,7 +616,8 @@ mod tests {
     #[test]
     fn ack_empty() {
         let f = Frame::AckFrame(AckFrame {
-            receiver_base_id: 0x010203,
+            frame_window_base_id: 0x010203,
+            packet_window_base_id: 0x040506,
             frame_acks: Vec::new(),
         });
         verify_consistent(&f);
@@ -778,70 +662,6 @@ mod tests {
         (0..size).map(|_| rand::random::<u8>()).collect::<Vec<_>>().into_boxed_slice()
     }
 
-    /*
-    #[test]
-    fn message_random() {
-        const NUM_ROUNDS: usize = 100;
-        const MAX_MESSAGES: usize = 100;
-        const MAX_DATA_SIZE: usize = 100;
-
-        for _ in 0..NUM_ROUNDS {
-            let mut messages = Vec::new();
-
-            for _ in 0 .. rand::random::<usize>() % MAX_MESSAGES {
-                let message = match rand::random::<u32>() % 4 {
-                    0 => Message::Datagram(Datagram {
-                        sequence_id: rand::random::<u32>(),
-                        channel_id: (rand::random::<usize>() % MAX_CHANNELS) as u8,
-                        window_parent_lead: rand::random::<u16>(),
-                        channel_parent_lead: rand::random::<u16>(),
-                        fragment_id: FragmentId {
-                            id: rand::random::<u16>(),
-                            last: rand::random::<u16>(),
-                        },
-                        data: random_data(MAX_DATA_SIZE),
-                    }),
-                    1 => Message::Datagram(Datagram {
-                        sequence_id: rand::random::<u32>(),
-                        channel_id: (rand::random::<usize>() % MAX_CHANNELS) as u8,
-                        window_parent_lead: rand::random::<u16>(),
-                        channel_parent_lead: rand::random::<u16>(),
-                        fragment_id: FragmentId {
-                            id: 0,
-                            last: 0,
-                        },
-                        data: random_data(MAX_DATA_SIZE),
-                    }),
-                    2 => Message::Ack(Ack {
-                        frames: FrameAck {
-                            base_id: rand::random::<u32>(),
-                            bitfield: rand::random::<u32>(),
-                            nonce: rand::random::<bool>(),
-                        },
-                        receiver_base_id: rand::random::<u32>(),
-                    }),
-                    3 => Message::Resync(Resync {
-                        sender_next_id: rand::random::<u32>(),
-                    }),
-                    _ => panic!()
-                };
-
-                messages.push(message);
-            }
-
-            let f = Frame::MessageFrame(MessageFrame {
-                sequence_id: rand::random::<u32>(),
-                nonce: rand::random::<bool>(),
-                messages: messages,
-            });
-
-            verify_consistent(&f);
-            verify_extra_bytes_fail(&f);
-            verify_truncation_fails(&f);
-        }
-    }
-    */
-
     #[test]
     fn data_random() {
         const NUM_ROUNDS: usize = 100;
@@ -899,9 +719,8 @@ mod tests {
 
         for _ in 0..NUM_ROUNDS {
             let f = Frame::SyncFrame(SyncFrame {
-                sequence_id: rand::random::<u32>(),
-                nonce: rand::random::<bool>(),
-                sender_next_id: rand::random::<u32>(),
+                next_frame_id: if rand::random::<u32>() % 5 != 0 { Some(rand::random::<u32>()) } else { None },
+                next_packet_id: if rand::random::<u32>() % 5 != 0 { Some(rand::random::<u32>()) } else { None },
             });
             verify_consistent(&f);
             verify_extra_bytes_fail(&f);
@@ -918,7 +737,7 @@ mod tests {
             let mut frame_acks = Vec::new();
 
             for _ in 0 .. rand::random::<usize>() % MAX_FRAME_ACKS {
-                let frame_ack = FrameAck {
+                let frame_ack = AckGroup {
                     base_id: rand::random::<u32>(),
                     bitfield: rand::random::<u32>(),
                     nonce: rand::random(),
@@ -928,7 +747,8 @@ mod tests {
             }
 
             let f = Frame::AckFrame(AckFrame {
-                receiver_base_id: rand::random::<u32>(),
+                frame_window_base_id: rand::random::<u32>(),
+                packet_window_base_id: rand::random::<u32>(),
                 frame_acks: frame_acks,
             });
 
